#!/usr/bin/env python3
from tensorflow.core.protobuf import config_pb2, rewriter_config_pb2, meta_graph_pb2
from tensorflow.core.framework import graph_pb2
from tensorflow.python.saved_model import tag_constants
from tensorflow.python.framework import importer, ops
from tensorflow.python.grappler import tf_optimizer
from tensorflow.python.training import saver

# For automatic output node deduction.
import graphsurgeon as gs
import argparse

def constfold(graphdef):
    graph = ops.Graph()
    with graph.as_default():
        outputs = [node.name for node in gs.StaticGraph(graphdef).graph_outputs]
        output_collection = meta_graph_pb2.CollectionDef()
        output_list = output_collection.node_list.value
        for output in outputs:
            output_list.append(output.encode("utf-8"))

        importer.import_graph_def(graphdef, name="")
        metagraph = saver.export_meta_graph(graph_def=graph.as_graph_def(add_shapes=True), graph=graph)
        metagraph.collection_def["train_op"].CopyFrom(output_collection)

    rewriter_config = rewriter_config_pb2.RewriterConfig()
    rewriter_config.optimizers.extend(["constfold"])
    rewriter_config.meta_optimizer_iterations = (rewriter_config_pb2.RewriterConfig.ONE)

    session_config = config_pb2.ConfigProto()
    session_config.graph_options.rewrite_options.CopyFrom(rewriter_config)
    return tf_optimizer.OptimizeGraph(session_config, metagraph, graph_id=b"tf_graph")

def main():
    parser = argparse.ArgumentParser("Folds constants in the provided frozen model")
    parser.add_argument("model_file", help="The frozen model in which to fold constants.")
    parser.add_argument("-o", "--output-file", help="The path at which to write the new graph, with constants folded.")
    args, _ = parser.parse_known_args()

    with open(args.model_file, 'rb') as f:
        graphdef = graph_pb2.GraphDef()
        graphdef.ParseFromString(f.read())

    folded_graph = constfold(graphdef)
    if args.output_file:
        with open(args.output_file, "wb") as f:
            f.write(folded_graph.SerializeToString())

if __name__ == '__main__':
    main()
